from .rest import RestClient


class Guardian(object):

    """Auth0 guardian endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)
    """

    def __init__(self, domain, token, telemetry=True):
        self.domain = domain
        self.client = RestClient(jwt=token, telemetry=telemetry)

    def _url(self, id=None):
        url = 'https://%s/api/v2/guardian' % self.domain
        if id is not None:
            return url + '/' + id
        return url

    def all_factors(self):
        """Retrieves all factors. Useful to check factor enablement and
             trial status.
        """

        return self.client.get(self._url('factors'))

    def update_factor(self, name, body):
        """Update Guardian factor
        Useful to enable / disable factor

        Args:
            name (str): Either push-notification or sms
            body (dict): Attributes to modify.
            See: https://auth0.com/docs/api/management/v2#!/Guardian/put_factors_by_name
        """
        url = self._url('factors/%s' % (name))
        return self.client.put(url, data=body)

    def update_templates(self, body):
        """Update enrollment and verification SMS templates.

        Useful to send custom messages on sms enrollment and verification

        Args:
            body (dict): Attributes to modify.
            See: https://auth0.com/docs/api/management/v2#!/Guardian/put_templates
        """

        return self.client.put(self._url('factors/sms/templates'), data=body)

    def get_templates(self):
        """Get enrollment and verification templates

        Retrieve both templates. Useful to check if a different template than
            default was set.
        """

        return self.client.get(self._url('factors/sms/templates'))

    def get_enrollment(self, id):
        """Retrieves an enrollment.
        Useful to check its type and related metadata.

        Args:
           id (str): The id of the device account to update
        """
        url = self._url('enrollments/%s' % (id))
        return self.client.get(url)

    def delete_enrollment(self, id):
        """Deletes an enrollment.

        Useful when you want to force re-enroll.

        Args:
           id (str): The id of the device account to update
        """
        url = self._url('enrollments/%s' % (id))
        return self.client.delete(url)

    def create_enrollment_ticket(self, body):
        """Creates an enrollment ticket for user_id

        A useful way to send an email to a user, with a link that lead to
            start the enrollment process

        Args:
            body (dict): Details of the user to send the ticket to.
            See: https://auth0.com/docs/api/management/v2#!/Guardian/post_ticket
        """
        return self.client.post(self._url('enrollments/ticket'), data=body)

    def get_factor_providers(self, factor_name, name):
        """Get Guardian factor providers.

        Returns provider configuration
        Args:
           factor_name (str): Either push-notification or sms
           name (str): Name of the provider
        """
        url = self._url('factors/%s/providers/%s' % (factor_name, name))
        return self.client.get(url)

    def update_factor_providers(self, factor_name, name, body):
        """Get Guardian factor providers.

        Returns provider configuration
        Args:
           factor_name (str): Either push-notification or sms
           name (str): Name of the provider
        """
        url = self._url('factors/%s/providers/%s' % (factor_name, name))
        return self.client.put(url, data=body)
